@@ -72,6 +72,7 @@ pub(crate) fn definitions(checker: &mut Checker) {
         Rule::UnderIndentation,
         Rule::UndocumentedMagicMethod,
         Rule::UndocumentedParam,
+        Rule::IncorrectSectionOrder,
         Rule::UndocumentedPublicClass,
         Rule::UndocumentedPublicFunction,
         Rule::UndocumentedPublicInit,
@@ -116,12 +117,15 @@ pub(crate) fn definitions(checker: &mut Checker) {
             })
     };
 
-    let definitions = std::mem::take(&mut checker.semantic.definitions);
     let mut overloaded_name: Option<&str> = None;
     for ContextualizedDefinition {
         definition,
         visibility,
-    } in definitions.resolve(exports.as_deref()).iter()
+    } in checker
+        .semantic
+        .definitions
+        .resolve(exports.as_deref())
+        .iter()
     {
         let docstring = docstrings::extraction::extract_docstring(definition);
 
@@ -286,6 +290,7 @@ pub(crate) fn definitions(checker: &mut Checker) {
                 Rule::MismatchedSectionUnderlineLength,
                 Rule::OverindentedSectionUnderline,
                 Rule::UndocumentedParam,
+                Rule::IncorrectSectionOrder,
             ]);
             if enforce_sections || enforce_pydoclint {
                 let section_contexts = pydocstyle::helpers::get_section_contexts(
