@@ -40,8 +40,7 @@ pub(crate) fn check_noqa(
         FileNoqaDirectives::extract(locator, comment_ranges, &settings.external, path);
 
     // Extract all `noqa` directives.
-    let mut noqa_directives =
-        NoqaDirectives::from_commented_ranges(comment_ranges, &settings.external, path, locator);
+    let mut noqa_directives = NoqaDirectives::from_commented_ranges(comment_ranges, path, locator);
 
     if file_noqa_directives.is_empty() && noqa_directives.is_empty() && suppressions.is_empty() {
         return Vec::new();
@@ -202,7 +201,6 @@ pub(crate) fn check_noqa(
 
                     if !(disabled_codes.is_empty()
                         && duplicated_codes.is_empty()
-                        && unknown_codes.is_empty()
                         && unmatched_codes.is_empty())
                     {
                         let edit = if valid_codes.is_empty() {
@@ -234,10 +232,6 @@ pub(crate) fn check_noqa(
                                         .iter()
                                         .map(|code| (*code).to_string())
                                         .collect(),
-                                    unknown: unknown_codes
-                                        .iter()
-                                        .map(|code| (*code).to_string())
-                                        .collect(),
                                     unmatched: unmatched_codes
                                         .iter()
                                         .map(|code| (*code).to_string())
@@ -272,7 +266,13 @@ pub(crate) fn check_noqa(
     if context.is_rule_enabled(Rule::InvalidRuleCode)
         && !exemption.enumerates(Rule::InvalidRuleCode)
     {
-        ruff::rules::invalid_noqa_code(context, &noqa_directives, locator, &settings.external);
+        ruff::rules::invalid_noqa_code(
+            context,
+            &file_noqa_directives,
+            &noqa_directives,
+            locator,
+            &settings.external,
+        );
     }
 
     ignored_diagnostics.sort_unstable();
