@@ -40,7 +40,7 @@ pub fn generate_noqa_edits(
 ) -> Vec<Option<Edit>> {
     let file_directives = FileNoqaDirectives::extract(locator, comment_ranges, external, path);
     let exemption = FileExemption::from(&file_directives);
-    let directives = NoqaDirectives::from_commented_ranges(comment_ranges, external, path, locator);
+    let directives = NoqaDirectives::from_commented_ranges(comment_ranges, path, locator);
     let comments = find_noqa_comments(
         diagnostics,
         locator,
@@ -277,24 +277,18 @@ impl<'a> FileNoqaDirectives<'a> {
                             vec![]
                         }
                         Directive::Codes(codes) => {
-                            codes.iter().filter_map(|code| {
-                                let code = code.as_str();
-                                // Ignore externally-defined rules.
-                                if external.iter().any(|external| code.starts_with(external)) {
-                                    return None;
-                                }
+                            codes
+                                .iter()
+                                .filter_map(|code| {
+                                    let code = code.as_str();
+                                    // Ignore externally-defined rules.
+                                    if external.iter().any(|external| code.starts_with(external)) {
+                                        return None;
+                                    }
 
-                                if let Ok(rule) = Rule::from_code(get_redirect_target(code).unwrap_or(code))
-                                {
-                                    Some(rule)
-                                } else {
-                                    #[expect(deprecated)]
-                                    let line = locator.compute_line_index(range.start());
-                                    let path_display = relativize_path(path);
-                                    warn!("Invalid rule code provided to `# ruff: noqa` at {path_display}:{line}: {code}");
-                                    None
-                                }
-                            }).collect()
+                                    Rule::from_code(get_redirect_target(code).unwrap_or(code)).ok()
+                                })
+                                .collect()
                         }
                     };
 
@@ -771,7 +765,7 @@ fn add_noqa_inner(
     let directives = FileNoqaDirectives::extract(locator, comment_ranges, external, path);
     let exemption = FileExemption::from(&directives);
 
-    let directives = NoqaDirectives::from_commented_ranges(comment_ranges, external, path, locator);
+    let directives = NoqaDirectives::from_commented_ranges(comment_ranges, path, locator);
 
     let comments = find_noqa_comments(
         diagnostics,
@@ -1082,7 +1076,6 @@ pub(crate) struct NoqaDirectives<'a> {
 impl<'a> NoqaDirectives<'a> {
     pub(crate) fn from_commented_ranges(
         comment_ranges: &CommentRanges,
-        external: &[String],
         path: &Path,
         locator: &'a Locator<'a>,
     ) -> Self {
@@ -1106,29 +1099,6 @@ impl<'a> NoqaDirectives<'a> {
                             );
                         }
                     }
-                    if let Directive::Codes(codes) = &directive {
-                        // Warn on invalid rule codes.
-                        for code in &codes.codes {
-                            // Ignore externally-defined rules.
-                            if !external
-                                .iter()
-                                .any(|external| code.as_str().starts_with(external))
-                            {
-                                if Rule::from_code(
-                                    get_redirect_target(code.as_str()).unwrap_or(code.as_str()),
-                                )
-                                .is_err()
-                                {
-                                    #[expect(deprecated)]
-                                    let line = locator.compute_line_index(range.start());
-                                    let path_display = relativize_path(path);
-                                    warn!(
-                                        "Invalid rule code provided to `# noqa` at {path_display}:{line}: {code}"
-                                    );
-                                }
-                            }
-                        }
-                    }
                     // noqa comments are guaranteed to be single line.
                     let range = locator.line_range(range.start());
                     directives.push(NoqaDirectiveLine {
@@ -1326,7 +1296,7 @@ mod tests {
     fn noqa_all() {
         let source = "# noqa";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1347,7 +1317,7 @@ mod tests {
     fn noqa_no_code() {
         let source = "# noqa:";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -1359,7 +1329,7 @@ mod tests {
     fn noqa_no_code_invalid_suffix() {
         let source = "# noqa: foo";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -1371,7 +1341,7 @@ mod tests {
     fn noqa_no_code_trailing_content() {
         let source = "# noqa:  # Foo";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -1383,7 +1353,7 @@ mod tests {
     fn malformed_code_1() {
         let source = "# noqa: F";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -1422,7 +1392,7 @@ mod tests {
     fn malformed_code_3() {
         let source = "# noqa: RUF001F";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             InvalidCodeSuffix,
         )
@@ -1492,7 +1462,7 @@ mod tests {
     fn noqa_all_case_insensitive() {
         let source = "# NOQA";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1625,7 +1595,7 @@ mod tests {
     fn noqa_all_no_space() {
         let source = "#noqa";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1704,7 +1674,7 @@ mod tests {
     fn noqa_all_multi_space() {
         let source = "#  noqa";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1837,7 +1807,7 @@ mod tests {
     fn noqa_all_leading_comment() {
         let source = "# Some comment describing the noqa # noqa";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1916,7 +1886,7 @@ mod tests {
     fn noqa_all_trailing_comment() {
         let source = "# noqa # Some comment describing the noqa";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1995,7 +1965,7 @@ mod tests {
     fn noqa_invalid_codes() {
         let source = "# noqa: unused-import, F401, some other code";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -2139,7 +2109,7 @@ mod tests {
     fn noqa_code_invalid_code_suffix() {
         let source = "# noqa: F401abc";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             InvalidCodeSuffix,
         )
@@ -2151,7 +2121,7 @@ mod tests {
     fn noqa_invalid_suffix() {
         let source = "# noqa[F401]";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             InvalidSuffix,
         )
@@ -2163,7 +2133,7 @@ mod tests {
     fn flake8_exemption_all() {
         let source = "# flake8: noqa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2184,7 +2154,7 @@ mod tests {
     fn flake8_noqa_no_code() {
         let source = "# flake8: noqa:";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2196,7 +2166,7 @@ mod tests {
     fn flake8_noqa_no_code_invalid_suffix() {
         let source = "# flake8: noqa: foo";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2208,7 +2178,7 @@ mod tests {
     fn flake8_noqa_no_code_trailing_content() {
         let source = "# flake8: noqa:  # Foo";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2220,7 +2190,7 @@ mod tests {
     fn flake8_malformed_code_1() {
         let source = "# flake8: noqa: F";
         let directive = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -2259,7 +2229,7 @@ mod tests {
     fn flake8_malformed_code_3() {
         let source = "# flake8: noqa: RUF001F";
         let directive = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             InvalidCodeSuffix,
         )
@@ -2271,7 +2241,7 @@ mod tests {
     fn ruff_exemption_all() {
         let source = "# ruff: noqa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2292,7 +2262,7 @@ mod tests {
     fn ruff_noqa_no_code() {
         let source = "# ruff: noqa:";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2304,7 +2274,7 @@ mod tests {
     fn ruff_noqa_no_code_invalid_suffix() {
         let source = "# ruff: noqa: foo";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2316,7 +2286,7 @@ mod tests {
     fn ruff_noqa_no_code_trailing_content() {
         let source = "# ruff: noqa:  # Foo";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2328,7 +2298,7 @@ mod tests {
     fn ruff_malformed_code_1() {
         let source = "# ruff: noqa: F";
         let directive = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -2367,7 +2337,7 @@ mod tests {
     fn ruff_malformed_code_3() {
         let source = "# ruff: noqa: RUF001F";
         let directive = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             InvalidCodeSuffix,
         )
@@ -2379,7 +2349,7 @@ mod tests {
     fn flake8_exemption_all_no_space() {
         let source = "#flake8:noqa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2400,7 +2370,7 @@ mod tests {
     fn ruff_exemption_all_no_space() {
         let source = "#ruff:noqa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2619,7 +2589,7 @@ mod tests {
     fn ruff_exemption_invalid_code_suffix() {
         let source = "# ruff: noqa: F401abc";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             InvalidCodeSuffix,
         )
@@ -2685,7 +2655,7 @@ mod tests {
     fn ruff_exemption_all_leading_comment() {
         let source = "# Leading comment # ruff: noqa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2706,7 +2676,7 @@ mod tests {
     fn ruff_exemption_all_trailing_comment() {
         let source = "# ruff: noqa # Trailing comment";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2754,7 +2724,7 @@ mod tests {
     fn ruff_exemption_all_trailing_comment_no_space() {
         let source = "# ruff: noqa# Trailing comment";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2775,7 +2745,7 @@ mod tests {
     fn ruff_exemption_all_trailing_comment_no_hash() {
         let source = "# ruff: noqa Trailing comment";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2823,7 +2793,7 @@ mod tests {
     fn flake8_exemption_all_case_insensitive() {
         let source = "# flake8: NoQa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2844,7 +2814,7 @@ mod tests {
     fn ruff_exemption_all_case_insensitive() {
         let source = "# ruff: NoQa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
