@@ -6,7 +6,6 @@ use ruff_text_size::Ranged;
 
 use crate::Violation;
 use crate::checkers::ast::Checker;
-use crate::preview::is_a003_class_scope_shadowing_expansion_enabled;
 use crate::rules::flake8_builtins::helpers::shadows_builtin;
 
 /// ## What it does
@@ -125,19 +124,12 @@ pub(crate) fn builtin_attribute_shadowing(
             //     def repeat(value: int, times: int) -> list[int]:
             //         return [value] * times
             // ```
-            // In stable, only consider references whose first non-type parent scope is the class
-            // scope (e.g., decorators, default args, and attribute initializers).
-            // In preview, also consider references from within the class scope.
             let consider_reference = |reference_scope_id: ScopeId| {
-                if is_a003_class_scope_shadowing_expansion_enabled(checker.settings()) {
-                    if reference_scope_id == scope_id {
-                        return true;
-                    }
-                }
-                checker
-                    .semantic()
-                    .first_non_type_parent_scope_id(reference_scope_id)
-                    == Some(scope_id)
+                reference_scope_id == scope_id
+                    || checker
+                        .semantic()
+                        .first_non_type_parent_scope_id(reference_scope_id)
+                        == Some(scope_id)
             };
 
             for reference in binding
