@@ -79,6 +79,7 @@ pub(crate) use stop_iteration_return::*;
 pub(crate) use subprocess_popen_preexec_fn::*;
 pub(crate) use subprocess_run_without_check::*;
 pub(crate) use super_without_brackets::*;
+pub(crate) use swap_with_temporary_variable::*;
 pub(crate) use sys_exit_alias::*;
 pub(crate) use too_many_arguments::*;
 pub(crate) use too_many_boolean_expressions::*;
@@ -190,6 +191,7 @@ mod stop_iteration_return;
 mod subprocess_popen_preexec_fn;
 mod subprocess_run_without_check;
 mod super_without_brackets;
+mod swap_with_temporary_variable;
 mod sys_exit_alias;
 mod too_many_arguments;
 mod too_many_boolean_expressions;
