@@ -3,12 +3,29 @@ use ruff_text_size::{Ranged, TextLen, TextRange, TextSize};
 
 use crate::Locator;
 use crate::checkers::ast::LintContext;
-use crate::noqa::{Code, Directive};
+use crate::fix::edits::delete_comment;
+use crate::noqa::{Code, Directive, FileNoqaDirectives};
 use crate::noqa::{Codes, NoqaDirectives};
+use crate::preview::is_file_level_invalid_rule_code_enabled;
 use crate::registry::Rule;
 use crate::rule_redirects::get_redirect_target;
 use crate::{AlwaysFixableViolation, Edit, Fix};
 
+#[derive(Debug, PartialEq, Eq)]
+pub(crate) enum InvalidRuleCodeKind {
+    Noqa,
+    Suppression,
+}
+
+impl InvalidRuleCodeKind {
+    fn as_str(&self) -> &str {
+        match self {
+            InvalidRuleCodeKind::Noqa => "`# noqa`",
+            InvalidRuleCodeKind::Suppression => "suppression",
+        }
+    }
+}
+
 /// ## What it does
 /// Checks for `noqa` codes that are invalid.
 ///
@@ -31,79 +48,108 @@ use crate::{AlwaysFixableViolation, Edit, Fix};
 /// ```
 ///
 /// ## Options
+///
+/// This rule will flag rule codes that are unknown to Ruff, even if they are
+/// valid for other tools. You can tell Ruff to ignore such codes by configuring
+/// the list of known "external" rule codes with the following option:
+///
 /// - `lint.external`
 #[derive(ViolationMetadata)]
-#[violation_metadata(preview_since = "0.11.4")]
+#[violation_metadata(stable_since = "0.15.0")]
 pub(crate) struct InvalidRuleCode {
     pub(crate) rule_code: String,
+    pub(crate) kind: InvalidRuleCodeKind,
+    pub(crate) whole_comment: bool,
 }
 
 impl AlwaysFixableViolation for InvalidRuleCode {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Invalid rule code in `# noqa`: {}", self.rule_code)
+        format!(
+            "Invalid rule code in {}: {}",
+            self.kind.as_str(),
+            self.rule_code
+        )
     }
 
     fn fix_title(&self) -> String {
-        "Remove the rule code".to_string()
+        if self.whole_comment {
+            format!("Remove the {} comment", self.kind.as_str())
+        } else {
+            format!("Remove the rule code `{}`", self.rule_code)
+        }
     }
 }
 
 /// RUF102 for invalid noqa codes
 pub(crate) fn invalid_noqa_code(
     context: &LintContext,
+    file_noqa_directives: &FileNoqaDirectives,
     noqa_directives: &NoqaDirectives,
     locator: &Locator,
     external: &[String],
 ) {
-    for line in noqa_directives.lines() {
-        let Directive::Codes(directive) = &line.directive else {
-            continue;
-        };
-
-        let all_valid = directive.iter().all(|code| code_is_valid(code, external));
+    let check_codes = |codes: &Codes<'_>| {
+        let all_valid = codes
+            .iter()
+            .all(|code| code_is_valid(code.as_str(), external));
 
         if all_valid {
-            continue;
+            return;
         }
 
-        let (valid_codes, invalid_codes): (Vec<_>, Vec<_>) = directive
+        let (valid_codes, invalid_codes): (Vec<_>, Vec<_>) = codes
             .iter()
-            .partition(|&code| code_is_valid(code, external));
+            .partition(|&code| code_is_valid(code.as_str(), external));
 
         if valid_codes.is_empty() {
-            all_codes_invalid_diagnostic(directive, invalid_codes, context);
+            all_codes_invalid_diagnostic(codes, invalid_codes, locator, context);
         } else {
             for invalid_code in invalid_codes {
-                some_codes_are_invalid_diagnostic(directive, invalid_code, locator, context);
+                some_codes_are_invalid_diagnostic(codes, invalid_code, locator, context);
             }
         }
+    };
+
+    if is_file_level_invalid_rule_code_enabled(context.settings()) {
+        for line in file_noqa_directives.lines() {
+            if let Directive::Codes(codes) = &line.parsed_file_exemption {
+                check_codes(codes);
+            }
+        }
+    }
+    for line in noqa_directives.lines() {
+        if let Directive::Codes(codes) = &line.directive {
+            check_codes(codes);
+        }
     }
 }
 
-fn code_is_valid(code: &Code, external: &[String]) -> bool {
-    let code_str = code.as_str();
-    Rule::from_code(get_redirect_target(code_str).unwrap_or(code_str)).is_ok()
-        || external.iter().any(|ext| code_str.starts_with(ext))
+pub(crate) fn code_is_valid(code: &str, external: &[String]) -> bool {
+    Rule::from_code(get_redirect_target(code).unwrap_or(code)).is_ok()
+        || external.iter().any(|ext| code.starts_with(ext))
 }
 
 fn all_codes_invalid_diagnostic(
     directive: &Codes<'_>,
     invalid_codes: Vec<&Code<'_>>,
+    locator: &Locator,
     context: &LintContext,
 ) {
-    context
-        .report_diagnostic(
-            InvalidRuleCode {
-                rule_code: invalid_codes
-                    .into_iter()
-                    .map(Code::as_str)
-                    .collect::<Vec<_>>()
-                    .join(", "),
-            },
-            directive.range(),
-        )
-        .set_fix(Fix::safe_edit(Edit::range_deletion(directive.range())));
+    let mut diagnostic = context.report_custom_diagnostic(
+        InvalidRuleCode {
+            rule_code: invalid_codes
+                .into_iter()
+                .map(Code::as_str)
+                .collect::<Vec<_>>()
+                .join(", "),
+            kind: InvalidRuleCodeKind::Noqa,
+            whole_comment: true,
+        },
+        directive.range(),
+    );
+    diagnostic.set_fix(Fix::safe_edit(delete_comment(directive.range(), locator)));
+    diagnostic.help("Add non-Ruff rule codes to the `lint.external` configuration option");
 }
 
 fn some_codes_are_invalid_diagnostic(
@@ -112,18 +158,20 @@ fn some_codes_are_invalid_diagnostic(
     locator: &Locator,
     context: &LintContext,
 ) {
-    context
-        .report_diagnostic(
-            InvalidRuleCode {
-                rule_code: invalid_code.to_string(),
-            },
-            invalid_code.range(),
-        )
-        .set_fix(Fix::safe_edit(remove_invalid_noqa(
-            codes,
-            invalid_code,
-            locator,
-        )));
+    let mut diagnostic = context.report_custom_diagnostic(
+        InvalidRuleCode {
+            rule_code: invalid_code.to_string(),
+            kind: InvalidRuleCodeKind::Noqa,
+            whole_comment: false,
+        },
+        invalid_code.range(),
+    );
+    diagnostic.set_fix(Fix::safe_edit(remove_invalid_noqa(
+        codes,
+        invalid_code,
+        locator,
+    )));
+    diagnostic.help("Add non-Ruff rule codes to the `lint.external` configuration option");
 }
 
 fn remove_invalid_noqa(codes: &Codes, invalid_code: &Code, locator: &Locator) -> Edit {
