@@ -11,6 +11,7 @@ pub(crate) use default_factory_kwarg::*;
 pub(crate) use duplicate_entry_in_dunder_all::*;
 pub(crate) use explicit_f_string_type_conversion::*;
 pub(crate) use falsy_dict_get_fallback::*;
+pub(crate) use float_equality_comparison::*;
 pub(crate) use function_call_in_dataclass_default::*;
 pub(crate) use if_key_in_dict_del::*;
 pub(crate) use implicit_classvar_in_dataclass::*;
@@ -50,6 +51,7 @@ pub(crate) use static_key_dict_comprehension::*;
 #[cfg(any(feature = "test-rules", test))]
 pub(crate) use test_rules::*;
 pub(crate) use unmatched_suppression_comment::*;
+pub(crate) use unnecessary_assign_before_yield::*;
 pub(crate) use unnecessary_cast_to_int::*;
 pub(crate) use unnecessary_iterable_allocation_for_first_element::*;
 pub(crate) use unnecessary_key_check::*;
@@ -80,6 +82,7 @@ mod default_factory_kwarg;
 mod duplicate_entry_in_dunder_all;
 mod explicit_f_string_type_conversion;
 mod falsy_dict_get_fallback;
+mod float_equality_comparison;
 mod function_call_in_dataclass_default;
 mod if_key_in_dict_del;
 mod implicit_classvar_in_dataclass;
@@ -121,6 +124,7 @@ mod suppression_comment_visitor;
 #[cfg(any(feature = "test-rules", test))]
 pub(crate) mod test_rules;
 mod unmatched_suppression_comment;
+mod unnecessary_assign_before_yield;
 mod unnecessary_cast_to_int;
 mod unnecessary_iterable_allocation_for_first_element;
 mod unnecessary_key_check;
