@@ -6,11 +6,12 @@
 //! types, and documentation. It supports multiple signatures for union types
 //! and overloads.
 
+use crate::Db;
 use crate::docstring::Docstring;
 use crate::goto::Definitions;
-use crate::{Db, find_node::covering_node};
 use ruff_db::files::File;
 use ruff_db::parsed::parsed_module;
+use ruff_python_ast::find_node::covering_node;
 use ruff_python_ast::token::TokenKind;
 use ruff_python_ast::{self as ast, AnyNodeRef};
 use ruff_text_size::{Ranged, TextRange, TextSize};
@@ -20,6 +21,7 @@ use ty_python_semantic::semantic_index::definition::Definition;
 use ty_python_semantic::types::ide_support::{
     CallSignatureDetails, call_signature_details, find_active_signature_from_details,
 };
+use ty_python_semantic::types::{ParameterKind, Type};
 
 // TODO: We may want to add special-case handling for calls to constructors
 // so the class docstring is used in place of (or inaddition to) any docstring
@@ -27,25 +29,29 @@ use ty_python_semantic::types::ide_support::{
 
 /// Information about a function parameter
 #[derive(Debug, Clone, PartialEq, Eq)]
-pub struct ParameterDetails {
+pub struct ParameterDetails<'db> {
     /// The parameter name (e.g., "param1")
     pub name: String,
     /// The parameter label in the signature (e.g., "param1: str")
     pub label: String,
+    /// The annotated type of the parameter. If no annotation was provided, this is `Unknown`.
+    pub ty: Type<'db>,
     /// Documentation specific to the parameter, typically extracted from the
     /// function's docstring
     pub documentation: Option<String>,
+    /// True if the parameter is positional-only.
+    pub is_positional_only: bool,
 }
 
 /// Information about a function signature
 #[derive(Debug, Clone, PartialEq, Eq)]
-pub struct SignatureDetails {
+pub struct SignatureDetails<'db> {
     /// Text representation of the full signature (including input parameters and return type).
     pub label: String,
     /// Documentation for the signature, typically from the function's docstring.
     pub documentation: Option<Docstring>,
     /// Information about each of the parameters in left-to-right order.
-    pub parameters: Vec<ParameterDetails>,
+    pub parameters: Vec<ParameterDetails<'db>>,
     /// Index of the parameter that corresponds to the argument where the
     /// user's cursor is currently positioned.
     pub active_parameter: Option<usize>,
@@ -53,18 +59,18 @@ pub struct SignatureDetails {
 
 /// Signature help information for function calls
 #[derive(Debug, Clone, PartialEq, Eq)]
-pub struct SignatureHelpInfo {
+pub struct SignatureHelpInfo<'db> {
     /// Information about each of the signatures for the function call. We
     /// need to handle multiple because of unions, overloads, and composite
     /// calls like constructors (which invoke both __new__ and __init__).
-    pub signatures: Vec<SignatureDetails>,
+    pub signatures: Vec<SignatureDetails<'db>>,
     /// Index of the "active signature" which is the first signature where
     /// all arguments that are currently present in the code map to parameters.
     pub active_signature: Option<usize>,
 }
 
 /// Signature help information for function calls at the given position
-pub fn signature_help(db: &dyn Db, file: File, offset: TextSize) -> Option<SignatureHelpInfo> {
+pub fn signature_help(db: &dyn Db, file: File, offset: TextSize) -> Option<SignatureHelpInfo<'_>> {
     let parsed = parsed_module(db, file).load(db);
 
     // Get the call expression at the given position.
@@ -119,6 +125,11 @@ fn get_call_expr(
         })?;
 
     // Find the covering node at the given position that is a function call.
+    // Note that we are okay with the range being anywhere within a call
+    // expression, even if it's not in the arguments portion of the call
+    // expression. This is because, e.g., a user can request signature
+    // information at a call site, and this should ideally work anywhere
+    // within the call site, even at the function name.
     let call = covering_node(root_node, token.range())
         .find_first(|node| {
             if !node.is_expr_call() {
@@ -166,11 +177,11 @@ fn get_argument_index(call_expr: &ast::ExprCall, offset: TextSize) -> usize {
 }
 
 /// Create signature details from `CallSignatureDetails`.
-fn create_signature_details_from_call_signature_details(
+fn create_signature_details_from_call_signature_details<'db>(
     db: &dyn crate::Db,
-    details: &CallSignatureDetails,
+    details: &CallSignatureDetails<'db>,
     current_arg_index: usize,
-) -> SignatureDetails {
+) -> SignatureDetails<'db> {
     let signature_label = details.label.clone();
 
     let documentation = get_callable_documentation(db, details.definition);
@@ -200,6 +211,8 @@ fn create_signature_details_from_call_signature_details(
         &signature_label,
         documentation.as_ref(),
         &details.parameter_names,
+        &details.parameter_kinds,
+        &details.parameter_types,
     );
     SignatureDetails {
         label: signature_label,
@@ -218,12 +231,14 @@ fn get_callable_documentation(
 }
 
 /// Create `ParameterDetails` objects from parameter label offsets.
-fn create_parameters_from_offsets(
+fn create_parameters_from_offsets<'db>(
     parameter_offsets: &[TextRange],
     signature_label: &str,
     docstring: Option<&Docstring>,
     parameter_names: &[String],
-) -> Vec<ParameterDetails> {
+    parameter_kinds: &[ParameterKind],
+    parameter_types: &[Type<'db>],
+) -> Vec<ParameterDetails<'db>> {
     // Extract parameter documentation from the function's docstring if available.
     let param_docs = if let Some(docstring) = docstring {
         docstring.parameter_documentation()
@@ -245,11 +260,17 @@ fn create_parameters_from_offsets(
 
             // Get the parameter name for documentation lookup.
             let param_name = parameter_names.get(i).map(String::as_str).unwrap_or("");
+            let is_positional_only = matches!(
+                parameter_kinds.get(i),
+                Some(ParameterKind::PositionalOnly { .. })
+            );
 
             ParameterDetails {
                 name: param_name.to_string(),
                 label,
+                ty: parameter_types[i],
                 documentation: param_docs.get(param_name).cloned(),
+                is_positional_only,
             }
         })
         .collect()
@@ -506,7 +527,8 @@ def ab(a: str): ...
             )
             .build();
 
-        assert_snapshot!(test.signature_help_render(), @r"
+        assert_snapshot!(test.signature_help_render(), @"
+
         ============== active signature =============
         (a: int) -> Unknown
         ---------------------------------------------
@@ -561,7 +583,8 @@ def ab(a: str):
             )
             .build();
 
-        assert_snapshot!(test.signature_help_render(), @r"
+        assert_snapshot!(test.signature_help_render(), @"
+
         ============== active signature =============
         (a: int) -> Unknown
         ---------------------------------------------
@@ -616,7 +639,8 @@ def ab(a: int):
             )
             .build();
 
-        assert_snapshot!(test.signature_help_render(), @r"
+        assert_snapshot!(test.signature_help_render(), @"
+
         ============== active signature =============
         (a: int, b: int) -> Unknown
         ---------------------------------------------
@@ -669,7 +693,8 @@ def ab(a: int):
             )
             .build();
 
-        assert_snapshot!(test.signature_help_render(), @r"
+        assert_snapshot!(test.signature_help_render(), @"
+
         ============== active signature =============
         (a: int, b: int) -> Unknown
         ---------------------------------------------
@@ -728,7 +753,8 @@ def ab(a: int, *, c: int):
             )
             .build();
 
-        assert_snapshot!(test.signature_help_render(), @r"
+        assert_snapshot!(test.signature_help_render(), @"
+
         ============== active signature =============
         (a: int, *, b: int) -> Unknown
         ---------------------------------------------
@@ -793,7 +819,8 @@ def ab(a: int, *, c: int):
             )
             .build();
 
-        assert_snapshot!(test.signature_help_render(), @r"
+        assert_snapshot!(test.signature_help_render(), @"
+
         ============== active signature =============
         (a: int, *, c: int) -> Unknown
         ---------------------------------------------
@@ -874,6 +901,85 @@ def ab(a: int, *, c: int):
         );
     }
 
+    #[test]
+    fn signature_help_generic_method_resolves_typevars() {
+        let test = cursor_test(
+            r#"
+        d: dict[str, int] = {"a": 1}
+        d.get(<CURSOR>
+        "#,
+        );
+
+        let result = test.signature_help().expect("Should have signature help");
+
+        // dict.get has multiple overloads; use the active signature
+        let active_idx = result
+            .active_signature
+            .expect("Should have an active signature");
+        let signature = &result.signatures[active_idx];
+
+        // The first parameter of dict.get is `key`, whose annotation is
+        // the TypeVar `_KT`. After TypeVar resolution at this call site,
+        // the parameter type should be `str` (not `_KT`).
+        let key_param = &signature.parameters[0];
+        assert_eq!(key_param.name, "key");
+        let type_display = format!("{}", key_param.ty.display(&test.db));
+        assert_eq!(type_display, "str");
+    }
+
+    #[test]
+    fn signature_help_generic_list_method_resolves_typevars() {
+        let test = cursor_test(
+            r#"
+        items: list[int] = [1, 2, 3]
+        items.append(<CURSOR>
+        "#,
+        );
+
+        let result = test.signature_help().expect("Should have signature help");
+        assert_eq!(result.signatures.len(), 1);
+
+        let signature = &result.signatures[0];
+
+        // list.append's parameter is typed as `_T`, which should resolve
+        // to `int` for a `list[int]`.
+        let object_param = &signature.parameters[0];
+        let type_display = format!("{}", object_param.ty.display(&test.db));
+        assert_eq!(type_display, "int");
+    }
+
+    #[test]
+    fn signature_help_generic_function_resolves_typevars() {
+        // This tests function-level TypeVar resolution, where the TypeVar
+        // is on the function itself (not inherited from a generic class).
+        // The specialization is inferred purely from the call arguments.
+        let test = cursor_test(
+            r#"
+        def pair[T](a: T, b: T) -> tuple[T, T]:
+            return (a, b)
+
+        def get_name() -> str: ...
+        pair(get_name(), <CURSOR>
+        "#,
+        );
+
+        let result = test.signature_help().expect("Should have signature help");
+        assert_eq!(result.signatures.len(), 1);
+
+        let signature = &result.signatures[0];
+
+        // `T` should be resolved to `str` from the first argument.
+        let a_param = &signature.parameters[0];
+        assert_eq!(a_param.name, "a");
+        let a_type = format!("{}", a_param.ty.display(&test.db));
+        assert_eq!(a_type, "str");
+
+        let b_param = &signature.parameters[1];
+        assert_eq!(b_param.name, "b");
+        let b_type = format!("{}", b_param.ty.display(&test.db));
+        assert_eq!(b_type, "str");
+    }
+
     #[test]
     fn signature_help_callable_object() {
         let test = cursor_test(
@@ -1173,7 +1279,7 @@ def ab(a: int, *, c: int):
     }
 
     impl CursorTest {
-        fn signature_help(&self) -> Option<SignatureHelpInfo> {
+        fn signature_help(&self) -> Option<SignatureHelpInfo<'_>> {
             crate::signature_help::signature_help(&self.db, self.cursor.file, self.cursor.offset)
         }
 
