@@ -33,11 +33,6 @@ impl ScopedPredicateId {
     fn is_terminal(self) -> bool {
         self >= Self::SMALLEST_TERMINAL
     }
-
-    #[cfg(test)]
-    pub(crate) fn as_u32(self) -> u32 {
-        self.0
-    }
 }
 
 impl Idx for ScopedPredicateId {
@@ -107,6 +102,10 @@ impl PredicateOrLiteral<'_> {
 pub(crate) struct CallableAndCallExpr<'db> {
     pub(crate) callable: Expression<'db>,
     pub(crate) call_expr: Expression<'db>,
+    /// Whether the call is wrapped in an `await` expression. If `true`, `call_expr` refers to the
+    /// `await` expression rather than the call itself. This is used to detect terminal `await`s of
+    /// async functions that return `Never`.
+    pub(crate) is_await: bool,
 }
 
 #[derive(Clone, Copy, Debug, Hash, PartialEq, Eq, salsa::Update, get_size2::GetSize)]
@@ -136,6 +135,7 @@ pub(crate) enum PatternPredicateKind<'db> {
     Value(Expression<'db>),
     Or(Vec<PatternPredicateKind<'db>>),
     Class(Expression<'db>, ClassPatternKind),
+    Mapping(ClassPatternKind),
     As(Option<Box<PatternPredicateKind<'db>>>, Option<Name>),
     Unsupported,
 }
