@@ -28,12 +28,10 @@ use crate::types::builder::RecursivelyDefined;
 use crate::types::class::{ClassType, KnownClass};
 use crate::types::constraints::{ConstraintSet, IteratorConstraintsExtension};
 use crate::types::generics::InferableTypeVars;
-use crate::types::relation::{
-    HasRelationToVisitor, IsDisjointVisitor, IsEquivalentVisitor, TypeRelation,
-};
+use crate::types::relation::{HasRelationToVisitor, IsDisjointVisitor, TypeRelation};
 use crate::types::{
     ApplyTypeMappingVisitor, BoundTypeVarInstance, FindLegacyTypeVarsVisitor, IntersectionType,
-    NormalizedVisitor, Type, TypeMapping, UnionBuilder, UnionType,
+    Type, TypeMapping, UnionBuilder, UnionType,
 };
 use crate::types::{Truthiness, TypeContext};
 use crate::{Db, FxOrderSet, Program};
@@ -128,11 +126,7 @@ impl TupleLength {
     }
 }
 
-/// # Ordering
-/// Ordering is based on the tuple's salsa-assigned id and not on its elements.
-/// The id may change between runs, or when the tuple was garbage collected and recreated.
 #[salsa::interned(debug, constructor=new_internal, heap_size=ruff_memory_usage::heap_size)]
-#[derive(PartialOrd, Ord)]
 pub struct TupleType<'db> {
     #[returns(ref)]
     pub(crate) tuple: TupleSpec<'db>,
@@ -223,18 +217,6 @@ impl<'db> TupleType<'db> {
         })
     }
 
-    /// Return a normalized version of `self`.
-    ///
-    /// See [`Type::normalized`] for more details.
-    #[must_use]
-    pub(crate) fn normalized_impl(
-        self,
-        db: &'db dyn Db,
-        visitor: &NormalizedVisitor<'db>,
-    ) -> Option<Self> {
-        TupleType::new(db, &self.tuple(db).normalized_impl(db, visitor))
-    }
-
     pub(super) fn recursive_type_normalized_impl(
         self,
         db: &'db dyn Db,
@@ -310,17 +292,6 @@ impl<'db> TupleType<'db> {
         )
     }
 
-    pub(crate) fn is_equivalent_to_impl(
-        self,
-        db: &'db dyn Db,
-        other: Self,
-        inferable: InferableTypeVars<'_, 'db>,
-        visitor: &IsEquivalentVisitor<'db>,
-    ) -> ConstraintSet<'db> {
-        self.tuple(db)
-            .is_equivalent_to_impl(db, other.tuple(db), inferable, visitor)
-    }
-
     pub(crate) fn is_single_valued(self, db: &'db dyn Db) -> bool {
         self.tuple(db).is_single_valued(db)
     }
@@ -427,11 +398,6 @@ impl<'db> FixedLengthTuple<Type<'db>> {
         }
     }
 
-    #[must_use]
-    fn normalized_impl(&self, db: &'db dyn Db, visitor: &NormalizedVisitor<'db>) -> Self {
-        Self::from_elements(self.0.iter().map(|ty| ty.normalized_impl(db, visitor)))
-    }
-
     fn recursive_type_normalized_impl(
         &self,
         db: &'db dyn Db,
@@ -592,22 +558,6 @@ impl<'db> FixedLengthTuple<Type<'db>> {
         }
     }
 
-    fn is_equivalent_to_impl(
-        &self,
-        db: &'db dyn Db,
-        other: &Self,
-        inferable: InferableTypeVars<'_, 'db>,
-        visitor: &IsEquivalentVisitor<'db>,
-    ) -> ConstraintSet<'db> {
-        ConstraintSet::from(self.0.len() == other.0.len()).and(db, || {
-            (self.0.iter())
-                .zip(&other.0)
-                .when_all(db, |(self_ty, other_ty)| {
-                    self_ty.is_equivalent_to_impl(db, *other_ty, inferable, visitor)
-                })
-        })
-    }
-
     fn is_single_valued(&self, db: &'db dyn Db) -> bool {
         self.0.iter().all(|ty| ty.is_single_valued(db))
     }
@@ -903,8 +853,11 @@ impl<'db> VariableLengthTuple<Type<'db>> {
                 let self_suffix_length = self.suffix_elements().len();
                 let suffix_overflow = self_suffix_length.saturating_sub(suffix_length);
                 let suffix_underflow = suffix_length.saturating_sub(self_suffix_length);
-                let prefix = (self.iter_prefix_elements().take(prefix_length))
-                    .chain(std::iter::repeat_n(self.variable(), prefix_underflow));
+                // Compute the variable element first, since underflow positions can
+                // receive any element that could appear in the variable portion.
+                // For example, `tuple[I0, *tuple[I1, ...], I2]` unpacked as
+                // `[a, b, *c]` means `b` could be `I1` (variable non-empty) or
+                // `I2` (variable empty, suffix shifts left), so it should be `I1 | I2`.
                 let variable = UnionType::from_elements_leave_aliases(
                     db,
                     self.iter_prefix_elements()
@@ -912,25 +865,15 @@ impl<'db> VariableLengthTuple<Type<'db>> {
                         .chain(std::iter::once(self.variable()))
                         .chain(self.iter_suffix_elements().take(suffix_overflow)),
                 );
-                let suffix = std::iter::repeat_n(self.variable(), suffix_underflow)
+                let prefix = (self.iter_prefix_elements().take(prefix_length))
+                    .chain(std::iter::repeat_n(variable, prefix_underflow));
+                let suffix = std::iter::repeat_n(variable, suffix_underflow)
                     .chain(self.iter_suffix_elements().skip(suffix_overflow));
                 Ok(VariableLengthTuple::mixed(prefix, variable, suffix))
             }
         }
     }
 
-    #[must_use]
-    fn normalized_impl(&self, db: &'db dyn Db, visitor: &NormalizedVisitor<'db>) -> TupleSpec<'db> {
-        let prefix = self
-            .prenormalized_prefix_elements(db, None)
-            .map(|ty| ty.normalized_impl(db, visitor));
-        let suffix = self
-            .prenormalized_suffix_elements(db, None)
-            .map(|ty| ty.normalized_impl(db, visitor));
-        let variable = self.variable().normalized_impl(db, visitor);
-        TupleSpec::Variable(Self::new(prefix, variable, suffix))
-    }
-
     fn recursive_type_normalized_impl(
         &self,
         db: &'db dyn Db,
@@ -1215,41 +1158,6 @@ impl<'db> VariableLengthTuple<Type<'db>> {
             }
         }
     }
-
-    fn is_equivalent_to_impl(
-        &self,
-        db: &'db dyn Db,
-        other: &Self,
-        inferable: InferableTypeVars<'_, 'db>,
-        visitor: &IsEquivalentVisitor<'db>,
-    ) -> ConstraintSet<'db> {
-        self.variable()
-            .is_equivalent_to_impl(db, other.variable(), inferable, visitor)
-            .and(db, || {
-                self.prenormalized_prefix_elements(db, None)
-                    .zip_longest(other.prenormalized_prefix_elements(db, None))
-                    .when_all(db, |pair| match pair {
-                        EitherOrBoth::Both(self_ty, other_ty) => {
-                            self_ty.is_equivalent_to_impl(db, other_ty, inferable, visitor)
-                        }
-                        EitherOrBoth::Left(_) | EitherOrBoth::Right(_) => {
-                            ConstraintSet::from(false)
-                        }
-                    })
-            })
-            .and(db, || {
-                self.prenormalized_suffix_elements(db, None)
-                    .zip_longest(other.prenormalized_suffix_elements(db, None))
-                    .when_all(db, |pair| match pair {
-                        EitherOrBoth::Both(self_ty, other_ty) => {
-                            self_ty.is_equivalent_to_impl(db, other_ty, inferable, visitor)
-                        }
-                        EitherOrBoth::Left(_) | EitherOrBoth::Right(_) => {
-                            ConstraintSet::from(false)
-                        }
-                    })
-            })
-    }
 }
 
 impl<'db> PyIndex<'db> for &VariableLengthTuple<Type<'db>> {
@@ -1414,17 +1322,6 @@ impl<'db> Tuple<Type<'db>> {
         }
     }
 
-    pub(crate) fn normalized_impl(
-        &self,
-        db: &'db dyn Db,
-        visitor: &NormalizedVisitor<'db>,
-    ) -> Self {
-        match self {
-            Tuple::Fixed(tuple) => Tuple::Fixed(tuple.normalized_impl(db, visitor)),
-            Tuple::Variable(tuple) => tuple.normalized_impl(db, visitor),
-        }
-    }
-
     pub(super) fn recursive_type_normalized_impl(
         &self,
         db: &'db dyn Db,
@@ -1502,26 +1399,6 @@ impl<'db> Tuple<Type<'db>> {
         }
     }
 
-    fn is_equivalent_to_impl(
-        &self,
-        db: &'db dyn Db,
-        other: &Self,
-        inferable: InferableTypeVars<'_, 'db>,
-        visitor: &IsEquivalentVisitor<'db>,
-    ) -> ConstraintSet<'db> {
-        match (self, other) {
-            (Tuple::Fixed(self_tuple), Tuple::Fixed(other_tuple)) => {
-                self_tuple.is_equivalent_to_impl(db, other_tuple, inferable, visitor)
-            }
-            (Tuple::Variable(self_tuple), Tuple::Variable(other_tuple)) => {
-                self_tuple.is_equivalent_to_impl(db, other_tuple, inferable, visitor)
-            }
-            (Tuple::Fixed(_), Tuple::Variable(_)) | (Tuple::Variable(_), Tuple::Fixed(_)) => {
-                ConstraintSet::from(false)
-            }
-        }
-    }
-
     pub(super) fn is_disjoint_from_impl(
         &self,
         db: &'db dyn Db,
@@ -1810,8 +1687,8 @@ impl<'db> Tuple<Type<'db>> {
         };
 
         TupleSpec::heterogeneous([
-            Type::IntLiteral(python_version.major.into()),
-            Type::IntLiteral(python_version.minor.into()),
+            Type::int_literal(python_version.major.into()),
+            Type::int_literal(python_version.minor.into()),
             int_instance_ty,
             release_level_ty,
             int_instance_ty,
@@ -2131,11 +2008,11 @@ impl<'db> TupleSpecBuilder<'db> {
                     && suffix.len() == var.suffix_elements().len()
                 {
                     for (existing, new) in prefix.iter_mut().zip(var.prefix_elements()) {
-                        *existing = IntersectionType::from_elements(db, [*existing, *new]);
+                        *existing = IntersectionType::from_two_elements(db, *existing, *new);
                     }
-                    *variable = IntersectionType::from_elements(db, [*variable, var.variable()]);
+                    *variable = IntersectionType::from_two_elements(db, *variable, var.variable());
                     for (existing, new) in suffix.iter_mut().zip(var.suffix_elements()) {
-                        *existing = IntersectionType::from_elements(db, [*existing, *new]);
+                        *existing = IntersectionType::from_two_elements(db, *existing, *new);
                     }
                     return Some(self);
                 }
