@@ -4,6 +4,7 @@ mod diagnostic;
 mod doc_highlights;
 mod document_symbols;
 mod execute_command;
+mod folding_range;
 mod goto_declaration;
 mod goto_definition;
 mod goto_type_definition;
@@ -26,6 +27,7 @@ pub(super) use diagnostic::DocumentDiagnosticRequestHandler;
 pub(super) use doc_highlights::DocumentHighlightRequestHandler;
 pub(super) use document_symbols::DocumentSymbolRequestHandler;
 pub(super) use execute_command::ExecuteCommand;
+pub(super) use folding_range::FoldingRangeRequestHandler;
 pub(super) use goto_declaration::GotoDeclarationRequestHandler;
 pub(super) use goto_definition::GotoDefinitionRequestHandler;
 pub(super) use goto_type_definition::GotoTypeDefinitionRequestHandler;
